import SwiftSyntax

/// Trim `Copyright ... All rights reserved.` header comments generated by Xcode.
open class HeaderCopyrightTrimmer: SyntaxRewriter
{
    private let _rewriter = FirstTokenRewriter { token in

        let descrpition = token.leadingTrivia.description

        guard descrpition.contains("Copyright")
            && descrpition.contains("All rights reserved.") else
        {
            return token
        }

        let pieces = token.leadingTrivia.pieces
        var index = token.leadingTrivia.startIndex

        while index < pieces.endIndex
            && pieces[index].isComment
        {
            let nextIndex = index.advanced(by: 1)
            guard nextIndex < pieces.endIndex else { break }

            let nextPiece = pieces[nextIndex]

            // Skip `.lineComment` + `\n` pairs.
            if nextPiece.numberOfNewlines > 0 {
                let success = pieces.formIndex(&index, offsetBy: 2, limitedBy: pieces.endIndex)
                if !success { // overflow
                    _ = pieces.formIndex(&index, offsetBy: 1, limitedBy: pieces.endIndex)
                    break
                }

                if nextPiece.numberOfNewlines > 1 {
                    break // Stop if multiple newlines.
                }
            }
            else {
                break
            }
        }

        let pieces2 = pieces[index..<token.leadingTrivia.endIndex]
        let token2 = token.withLeadingTrivia(.init(pieces: Array(pieces2)))
        return token2
    }

    open override func visit(_ syntax: SourceFileSyntax) -> Syntax
    {
        return self._rewriter.visit(syntax)
    }
}
